/*

Introduction
------------
This script can be used to create a bootable CD with enough facilities
to fully manage and recover your system.

It contains support for LVM, HPFS, JFS, FAT32 and CD.  The total space
occupied is less than five MB!

Personally I use it on the CD-RWs I burn my system backups to.
Recovering/reinstalling the system just takes booting-off this CD,
optionally managing (lvm) and formatting volumes, and unzipping the
backup-zip files that are also on the CD!

Present settings assume an IDE only setup, instuctions for adding SCSI
support are included in the configuration file.  If you need extra
space on the boot image (for instance if you added SCSI support),
instructions for moving some files to the CD are included as well.

The script assumes you use RSJ for burning CDs and that you have
already attached the CD-R(W).

It furthermore assumes you use the DANI drivers for accessing IDE and
ATAPI devices as these are the most stable. This can be configured,
however.


Installation 
------------
Installation is simple, just copy the files to the correct locations:
    bootCD.cmd   should go to a directory on the PATH
    bootCD.cfg   should go to a directory on the DPATH
    daniboot.flt should go to a directory on the DPATH

Instead of copying the files to locations on the PATH/DPATH, a
specific directory where all files reside will of course also work
(just make sure the command is invoked from that directory...)

See below for instructions on where to get the required files


Configuration
------------- 
Configuration of the script is via the bootCD.cfg file.  The options
are all commented.  Work through them top to bottom, most likely you
will only need/want to change a few of the 'basic' ones.  You'll
probably not going to neet to bother with the 'advanced' ones (unless
you don't use the DANI drivers AND you know what you are doing...



Obtaining required files
------------------------
All required files can be obtained from Hobbes:
    savedskf.exe: search for loaddf
    daniboot.flt: search for daniboot
    danis506.add: search for danis506
    daniatap.flt: search for daniatapi
    fat32:        search for os2fat32
    virtual disk: search for vfdisk

Another option for a virtual disk is Super Virtual Disk
This is shareware and can be obtained from bmtmicro

For info about RSJ, look at their website


Thanks
------
Thanks go to Allan Dermody for providing CD_BOOT.FLT and the
first insight into creating bootable CD's

Thanks also go to Alfrede Fernandez Diaz for writing an excelent
article on how to create a bootable CD in OS2Voice, July 2000
http://www.os2voice.org/VNL/past_issues/VNL0700H/vnewsf2.htm

More thanks go to Daniela Engert for writing the DANI suite of
device drivers and filters.

Enjoy!

Contact information:
--------------------
Author:
    Name:    Hayo Baan
    e-mail:  hrbaan@xs4all.nl
    updates: http://www.xs4all.nl/~hrbaan

RSJ:
    Website: http://www.rsj.de
    
BMT-Micro:
    Website: http://www.bmtmicro.com

Hobbes:
    Website: http://hobbes.nmsu.edu


Version History
--------------- 
v1.3 22/08/2002: Added USB support
                 Fixed bug with SHELLFILE variable not respected
                 Renamed option IDECD to ATAPI (more correct)
                 Renamed option HDDFALLBACK to DASDFALLBACK (more
                   correct)
                 Now possible to specify options for IFS and BASEDEV
                   lines
                 Added comment why there is no autocheck for IFSs
                 Better option handling, not case sensitive anymore,
                   specifying 'Y' is enough
                 Cache sizes for HPFS and JFS now 512KB by default
                 Added VERBOSE option with this to "YES" all command output
                   is shown.
                 Added option to pause the script right before writing the
                   image to CD.  You can use this to check and or correct
                   the content of the boot image.
                 

v1.2 20/08/2002: Now a REXX script
                 Added configuration options
                 CONFIG.SYS is now created automatically by the script
                   and respects set options.
                 Added notes concerning virtual disk software
                 Added contact information

v1.1 21/02/2002: Now using daniboot.flt instead of cdboot.flt

v1.0 16/02/2002: Original version

*/

"@echo off"

/* Load the rexUtil functions                                              */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

/* String variable to insert a newline in a character string               */
CRLF=d2c(13)||d2c(10)

/* Read the configuration parameters                                       */
rc=readConfig()

if isSet(VERBOSE) then
    IGNOREOUTPUT=""
else
    IGNOREOUTPUT="> /dev/nul 2>&1"

/***************************************************************************/
/**** Boot Image ***********************************************************/
/***************************************************************************/

/* Format the virtual disk */
"format "VIRTUALDRIVE" /fs:fat /once /v:CD-BOOT "IGNOREOUTPUT

/* Install system files */
SYSINSTX VIRTUALDRIVE" "IGNOREOUTPUT

/* OS/2 directory tree */
"mkdir "VIRTUALDRIVE"\OS2"
"mkdir "VIRTUALDRIVE"\OS2\BOOT"
"mkdir "VIRTUALDRIVE"\OS2\DLL"

/* Files to copy to root directory of boot image */
rc=copyFiles(VITALROOTFILES, VIRTUALDRIVE"\")

/* Files to copy to os2 directory of boot image */
FILES=VITALFILES
if \ isSet(MAKEROOM) then FILES=FILES VITALFILES_ SHELLFILE
rc=copyFiles(FILES, VIRTUALDRIVE"\os2")

/* Files to copy to os2\boot directory of boot image */
BOOTFILES=VITALBOOTFILES BOOTFLTFILE
if isSet(FLOPPY)       then BOOTFILES=BOOTFILES FLOPPYADDFILE
if isSet(DASDFALLBACK) then BOOTFILES=BOOTFILES DASDFALLBACKFILE
if isSet(CDROM)        then BOOTFILES=BOOTFILES CDROMDMDFILE
if isSet(IDE)          then BOOTFILES=BOOTFILES IDEADDFILE
if isSet(SCSI)         then BOOTFILES=BOOTFILES SCSIADDFILE
if isSet(ATAPI)        then BOOTFILES=BOOTFILES ATAPIFLTFILE
if isSet(ASPI)         then BOOTFILES=BOOTFILES ASPIDMDFILE
if isSet(CDFS)         then BOOTFILES=BOOTFILES CDFSIFSFILE
if isSet(FAT)          then BOOTFILES=BOOTFILES FATIFSFILE
if isSet(HPFS)         then BOOTFILES=BOOTFILES HPFSIFSFILE
if isSet(JFS)          then BOOTFILES=BOOTFILES JFSIFSFILE
if isSet(LVM)          then BOOTFILES=BOOTFILES LVMDMDFILE
if isSet(USB) then do
    BOOTFILES=BOOTFILES USBDFILE USBHIDFILE
    if USBOHCI>0 then BOOTFILES=BOOTFILES USBOHCIFILE
    if USBUHCI>0 then BOOTFILES=BOOTFILES USBUHCIFILE
    if USBEHCI>0 then BOOTFILES=BOOTFILES USBEHCIFILE
    if isSet(USBCDROM)   then BOOTFILES=BOOTFILES USBCDROMFILE
    if isSet(USBSTORAGE) then BOOTFILES=BOOTFILES USBSTORAGEFILE
end
rc=copyFiles(BOOTFILES, VIRTUALDRIVE"\os2\boot")

/* Files to copy to os2\dll directory of boot image */
rc=copyFiles(VITALDLLFILES, VIRTUALDRIVE"\os2\dll")

/* Unicode support files */
if isSet(UNICODE) then do
  /* Language directory tree */
  "mkdir "VIRTUALDRIVE"\LANGUAGE"
  "mkdir "VIRTUALDRIVE"\LANGUAGE\CODEPAGE"
  "mkdir "VIRTUALDRIVE"\LANGUAGE\"UNILANGDIR
  "mkdir "VIRTUALDRIVE"\LANGUAGE\FONT"
  "mkdir "VIRTUALDRIVE"\LANGUAGE\KEYBOARD"
  "mkdir "VIRTUALDRIVE"\LANGUAGE\LOCALE"
  rc=copyFiles(OSBOOTDRIVE"\os2\boot\unicode.sys",         VIRTUALDRIVE"\os2\boot")
  rc=copyFiles(OSBOOTDRIVE"\language\codepage\"UNICP,      VIRTUALDRIVE"\language\codepage")
  rc=copyFiles(OSBOOTDRIVE"\language\"UNILANGDIR"\ehx*.*", VIRTUALDRIVE"\language\"UNILANGDIR)
  rc=copyFiles(OSBOOTDRIVE"\language\keyboard\"UNIKEYB,    VIRTUALDRIVE"\language\keyboard")
  rc=copyFiles(OSBOOTDRIVE"\language\locale\user.loc",     VIRTUALDRIVE"\language\locale")
end


/***************************************************************************/
/**** Create config.sys file ***********************************************/
/***************************************************************************/

/* Drive letter for files that can reside on boot image or on CD */
DRIVELETTER="a:"
if isSet(MAKEROOM) then DRIVELETTER="s:"

/* CD-ROM = S: */
rc=configLine("RESERVEDRIVELETTER=R")

/* File systems */
if isSet(CDFS) then rc=configLine("IFS=\os2\boot\"fileName(CDFSIFSFILE) CDFSOPTIONS)
if isSet(FAT)  then rc=configLine("IFS=\os2\boot\"fileName(FATIFSFILE)  FATOPTIONS)
if isSet(HPFS) then rc=configLine("IFS=\os2\boot\"fileName(HPFSIFSFILE) HPFSOPTIONS)
if isSet(JFS)  then rc=configLine("IFS=\os2\boot\"fileName(JFSIFSFILE)  JFSOPTIONS)

/* Base device drivers */
rc=configLine("BASEDEV=ibmkbd.sys")
rc=configLine("BASEDEV=os2dasd.dmd")

if isSet(FLOPPY)       then rc=configLine("BASEDEV="fileName(FLOPPYADDFILE)    FLOPPYOPTIONS)
if isSet(IDE)          then rc=configLine("BASEDEV="fileName(IDEADDFILE)       IDEOPTIONS)
if isSet(SCSI)         then rc=configLine("BASEDEV="fileName(SCSIADDFILE)      SCSIOPTIONS)
if isSet(ATAPI)        then rc=configLine("BASEDEV="fileName(ATAPIFLTFILE)     ATAPIOPTIONS)
if isSet(DASDFALLBACK) then rc=configLine("BASEDEV="fileName(DASDFALLBACKFILE) DASDFALLBACKOPTIONS)
if isSet(LVM)          then rc=configLine("BASEDEV="fileName(LVMDMDFILE)       LVMOPTIONS)

rc=configLine("BASEDEV="fileName(BOOTFLTFILE) BOOTOPTIONS)

/* Codepages and keyboard layout */
rc=configLine("COUNTRY="COUNTRY",\os2\country.sys")
rc=configLine("CODEPAGE="CODEPAGE)
rc=configLine("DEVINFO=KBD,"KEYBOARD",\os2\boot\keyboard.dcp")
rc=configLine("DEVINFO=SCR,VGA,\os2\boot\viotbl.dcp")

/* Unicode */
if isSet(UNICODE) then do
  rc=configLine("SET LANG="UNILANG)
  rc=configLine("SET ULSPATH=\language")
  rc=configLine("DEVICE=\os2\boot\unicode.sys")
end

/* Device Drivers */
if isSet(CDROM) then rc=configLine("DEVICE=\os2\boot\"fileName(CDROMDMDFILE))
if isSet(ASPI)  then rc=configLine("DEVICE=\os2\boot\"fileName(ASPIFILE))

/* USB */
if isSet(USB) then do
    do USBOHCI
        rc=configLine("BASEDEV="fileName(USBOHCIFILE) USBOHCIOPTIONS)
    end
    do USBUHCI
        rc=configLine("BASEDEV="fileName(USBUHCIFILE) USBUHCIOPTIONS)
    end
    do USBEHCI
        rc=configLine("BASEDEV="fileName(USBEHCIFILE) USBEHCIOPTIONS)
    end
    rc=configLine("BASEDEV="fileName(USBDFILE)   USBDOPTIONS)
    rc=configLine("BASEDEV="fileName(USBHIDFILE) USBHIDOPTIONS)
    if isSet(USBKEYB)    then rc=configLine("DEVICE=s:\os2\"fileName(USBKEYBFILE)    USBKEYBOPTIONS)
    if isSet(USBCOM)     then rc=configLine("DEVICE=s:\os2\"fileName(USBCOMFILE)     USBCOMOPTIONS)
    if isSet(USBMOUSE)   then rc=configLine("DEVICE=s:\os2\"fileName(USBMOUSEFILE)   USBMOUSEOPTIONS)
    if isSet(USBPRINTER) then rc=configLine("DEVICE=s:\os2\"fileName(USBPRINTERFILE) USBPRINTEROPTIONS)
    if isSet(USBCDROM)   then rc=configLine("BASEDEV="fileName(USBCDROMFILE)   USBCDROMOPTIONS)
    if isSet(USBSTORAGE) then rc=configLine("BASEDEV="fileName(USBSTORAGEFILE) USBSTORAGEOPTIONS)
end

/* Settings */
rc=configLine("MEMMAN=NOSWAP,PROTECT")
rc=configLine("IOPL=YES")
rc=configLine("PROTECTONLY=YES")
rc=configLine("BUFFERS=30")
rc=configLine("DISKCACHE=48,LW")

/* Test device driver */
rc=configLine("DEVICE="DRIVELETTER"\os2\testcfg.sys")

/* Pointer support */
rc=configLine("DEVICE="DRIVELETTER"\os2\pointdd.sys")
rc=configLine("DEVICE="DRIVELETTER"\os2\mouse.sys")

/* Command environment */
rc=configLine("PROTSHELL="DRIVELETTER"\os2\"fileName(SHELLFILE))
rc=configLine("SET OS2_SHELL="DRIVELETTER"\os2\"fileName(SHELLFILE))
rc=configLine("SET COMSPEC="DRIVELETTER"\os2\"fileName(SHELLFILE))
rc=configLine("SET PROMPT=$P$$$s")
rc=configLine("SET KEYS=ON")
rc=configLine("LIBPATH=.;a:\;a:\os2\dll;a:\os2\boot;s:\os2\dll")
rc=configLine("SET PATH=a:\;a:\os2;a:\os2\boot;s:\os2;")
rc=configLine("SET DPATH=a:\;a:\os2;a:\os2\boot;s:\os2;")

/* Additional config.sys lines */
rc=configLine(ADDITIONALCFG)

/* Close config.sys */
rc=lineout(VIRTUALDRIVE"\config.sys")

/* Create the boot image ------------------------------------------------- */

if isSet(PAUSEBEFOREBOOTIMAGE) then do
    "echo Pausing before writing boot image." IGNOREOUTPUT
    "pause"
end

BOOTCDIMAGE=value('TEMP',,'OS2ENVIRONMENT')"\bootcd.img"
"savedskf /a/d" VIRTUALDRIVE BOOTCDIMAGE IGNOREOUTPUT
"copy" BOOTCDIMAGE WRITERDRIVE
/* Mark the image as bootable */
"cdboot" WRITERDRIVE"\"fileName(BOOTCDIMAGE) IGNOREOUTPUT
"del" BOOTCDIMAGE IGNOREOUTPUT

/***************************************************************************/
/**** Support Files on CD **************************************************/
/***************************************************************************/

/* Directory tree */
"mkdir "WRITERDRIVE"\OS2"
"mkdir "WRITERDRIVE"\OS2\DLL"

/* Files to copy to os2 directory of CD */
FILES=EXTRAFILES
if isSet(CDFS)     then FILES=FILES CDFSFILES
if isSet(FAT)      then FILES=FILES FATFILES
if isSet(HPFS)     then FILES=FILES HPFSFILES
if isSet(JFS)      then FILES=FILES JFSFILES
if isSet(LVM)      then FILES=FILES LVMFILES
if isSet(MAKEROOM) then FILES=FILES VITALFILES_ SHELLFILE
if isSet(USB) then do
    if isSet(USBKEYB)    then FILES=FILES USBKEYBFILE
    if isSet(USBCOM)     then FILES=FILES USBCOMFILE
    if isSet(USBMOUSE)   then FILES=FILES USBMOUSEFILE
    if isSet(USBPRINTER) then FILES=FILES USBPRINTERFILE
end
rc=copyFiles(FILES, WRITERDRIVE"\os2")

/* Files to copy to os2\dll directory of CD */
DLLS=EXTRADLLS
if isSet(CDFS)    then DLLS=DLLS CDFSDLLS
if isSet(FAT)     then DLLS=DLLS FATDLLS
if isSet(HPFS)    then DLLS=DLLS HPFSDLLS
if isSet(JFS)     then DLLS=DLLS JFSDLLS
if isSet(LVM)     then DLLS=DLLS LVMDLLS
if isSet(UNICODE) then DLLS=DLLS OSBOOTDRIVE"\os2\dll\uconv.dll"
rc=copyFiles(DLLS,  WRITERDRIVE"\os2\dll")

exit


/***************************************************************************/
/**** Functions ************************************************************/
/***************************************************************************/

copyFiles:
    SOURCEFILES=ARG(1)
    DESTINATION=ARG(2)
    do while words(SOURCEFILES)>0
        SOURCEFILE=word(SOURCEFILES,1)
        "xcopy /h/t/r" SOURCEFILE DESTINATION IGNOREOUTPUT
        if rc<>0 then do
            select
                when rc=1 then ERR="File not found"
                when rc=2 then ERR="File errors"
                when rc=3 then ERR="Interrupted"
                when rc=4 then ERR="Not enough room for file"
                otherwhise ERR="Unknown error rc="rc
            end
            "echo Copying" SOURCEFILE "failed:" ERR ">&2"
        end
        SOURCEFILES=DELWORD(SOURCEFILES,1,1)
    end
    return rc

configLine:
    rc=lineout(VIRTUALDRIVE"\config.sys",ARG(1))
    return rc

fileName:
    return filespec("NAME", ARG(1))

readConfig:
    CONFIGFILE=SysSearchPath("DPATH","bootcd.cfg")
    STATEMENT=""
    LINENR=0
    do while lines(CONFIGFILE)>0
        LINE=strip(linein(CONFIGFILE))
        LINENR=LINENR+1
        if length(LINE)>0 & left(LINE,2)<>"/*" then do
            CONT=0
            if right(LINE,1)="," then do
                CONT=1
                LINE=left(LINE,length(LINE)-1)
            end
            STATEMENT=strip(STATEMENT LINE)
            if CONT=0 then do
                interpret STATEMENT
                STATEMENT=""
            end
        end
    end
    return 0

isSet:
    return (left(translate(ARG(1)),1)="Y")
